 /*********************************************************************

   CLRBOOK.CLASS

   Rick Seeger
   Interport Communications
   1133 Broadway, Suite 1004
   New York, NY 10021
   11/23/96

   Applet Parameters
   <paints>     : Number of paints per palette (4-16, default 8)
   <mask>       : Bit mask enabling palettes (1-63, default 32)
                    1 - basic
                    2 - pastel
                    4 - water color
                    8 - oil
                   16 - charcoal
                   32 - random 
   <file>       : Filename of the line drawing graphic (required)
   <imgwidth>   : Width of line drawing (default 640)
   <imgheight>  : Height of line drawing (default 480)
   <background> : Background color as RRGGBB (default FFFFFF)
   <foreground> : Foreground color as RRGGBB (default 000000)
   <palvertical>: Set to value > 0 if you wish the palette to be
                  drawn vertically instead of horizontally


   Note: You must set the validHost variable to the name of the webserver
   that hosts this applet or else it will not run.

   *********************************************************************/

import java.util.Random;
import java.lang.Math;
import java.applet.*;
import java.awt.image.*;
import java.awt.*;
import java.net.*;
import java.awt.event.*;

 public class clrbook extends Applet {

	 // security
	 private final boolean enableSecurity = false;
	 private final String validHost = "www.weeklyreader.com";
	 

	 // aesthetic constants
	 
	 private final int paintHeight = 50;
	 private final int paintSpacing = 5;
	 private final int paintEdge = 4;
	 private final int palMargin = 10;
	 private final int frameThickness = 5;
	 private final int frameColor = 0xFFFFFFFF;
	 private final int frameColor2 = 0xFF000000;
	 private int palBkgColor = 0xFFFFFFFF;
	 private final int paintEdgeColor = 0xFFFFFFFF;
	 private final int paintSelectedColor = 0xFF000000;
	 private final int WHITE = 0xFFFFFFFF;
	 private final int BLACK = 0xFF000000;
	 private  int buttonWidth = 60;
	 private  int buttonHeight = 20;
	 private final int buttonXMargin = 5;
	 private final int boxColor = 0xFF808080;
	 private final int numPalettes = 6;
	 
	 // internal variables
	 
	 private int palette[];                        
	 private Image imgFile, imgScreen, imgPal;
	 private int pixFile[], pixScreen[];           // pixel maps
	 private int pixPal[], pixPaint[]; 
	 private int imageSize, selectedColor=0;
	 private int selectedPalette = 0;
	 private int palHeight, palWidth;
	 private ImageButton more, clear;
	 private doButtons buttonAction = new doButtons();
	 private boolean firstPass = true;
	 private boolean secondPass = false;
	 private int paintWidth;
	 private String filename;
	 private int boxWidth;
	 private boolean authorized = false;
	 
	 // applet parameters and default values

	 private int numPaints=8;
	 private int paletteMask=32;
	 private int imgWidth=640;
	 private int imgHeight=480;
	 private String palPos="NORTH";
	 
	 // colors in order of importance
	 
	 private final int palStatic[] = {
		 0xFF000000,        // black
		 0xFFFFFFFF,        // white

		 0xFFFF0000,        // red
		 0xFF00FF00,        // green
		 0xFF0000FF,        // blue

		 0xFF00FFFF,        // cyan
		 0xFFFF00FF,        // magenta
		 0xFFFFFF00,        // yellow

		 0xFF8000FF,        // purple
		 0xFFFF8000,        // tangerine
		 0xFF0080FF,        // skyblue

		 0xFF00FF80,        // sea green
		 0xFF80FF00,        // deep red
		 0xFFFF0080,        // green algae

		 0xFF404040,        // dark grey
		 0xFFA0A0A0         // light grey
	 };
	 


	 // setDynamicPalette() 
	 // Redefines the paint palette colors based on the current 
	 // value of selectedPalette. 

	 private void setDynamicPalette() {
		 int i;

		 // the first four palettes are shifted versions of the static palette
		 if (selectedPalette < 4) {
			 double distance=0.0;
			 float center=0, red, green, blue;

			 switch(selectedPalette) {
			 case 0:            // basic
				 center = 0;
				 distance = 0.0;
				 break;
			 case 1:            // pastel
				 center = 255;    
				 distance = 0.5;
				 break;
			 case 2:            // water color
				 center = 128;
				 distance = 0.5;
				 break;
			 case 3:            // oil based
				 center = 0;
				 distance = 0.4;
				 break;
			 }
			 
			 // move all colors a certain distance to the center
			 for(i=0;i<numPaints;i++) {
				 red = (palStatic[i] >> 16) & 0x000000FF;
				 green = (palStatic[i] >> 8) & 0x000000FF;
				 blue = palStatic[i] & 0x000000FF;
				 palette[i] = (255 << 24) | (((int)(red+((center-red)*distance))) << 16) |
							  (((int)(green+((center-green)*distance))) << 8) | 
							  ((int)(blue+((center-blue)*distance)));
			 }
		 }

		 else if (selectedPalette==4) {     // charcoal
			 double grey;
			 int shade;
			 for(grey=0;grey<numPaints;grey++) {
				 shade = (int)((grey*255.0)/((double)(numPaints-1.0)));
				 palette[(int)grey] = (255 << 24) | (shade << 16) | (shade << 8) | shade;
			 }
		 }
		 
		 else if (selectedPalette==5) {     // random
			 Random r = new Random();
			 for(i=0;i<numPaints;i++) 
				 palette[i] = (r.nextInt() | 0xFF000000);
		 }
	 }

	 

	 // createPaletteImage() 
	 // Redefines the image representing the palette based on current
	 // color settings.
	 
	 private void createPaletteImage() {
		 int p,x,i,j,dest;

		 for(p=0,x=palMargin;p<numPaints;p++,x+=(paintWidth+paintSpacing)) {
			 for(i=0;i<paintWidth;i++) {      // overlay paint oval pixel map
				 for(j=0;j<paintHeight;j++) {
					 if (palPos=="EAST") dest = ((i+x)*palHeight)+j+palMargin;
					 else dest = ((j+palMargin)*palWidth)+x+i;

					 if (pixPaint[(j*paintWidth)+i]==1)           // inside paint oval
						 pixPal[dest] = palette[p];
					 else if (pixPaint[(j*paintWidth)+i]==2) {    // outer oval edge
						 if (selectedColor==p) 
							 if (paintSelectedColor == 0) {
								pixPal[dest] = palette[p];
							 } else {
								pixPal[dest] = paintSelectedColor;
							 }
						 else 
							 pixPal[dest] = paintEdgeColor;
					 }
					 else                                         // outside oval
						 pixPal[dest] = palBkgColor;
				 }
			 }
		 }
		 if (palPos=="EAST") 
			 imgPal = createImage(new MemoryImageSource(palHeight,palWidth,pixPal,0,palHeight));
		 else
			 imgPal = createImage(new MemoryImageSource(palWidth,palHeight,pixPal,0,palWidth));
	 }



	 // initPalette() 
	 // Initializes the bit-map representing the palette.

	 private void initPalette() {
		 double x,y,tx,ty,a,b;
		 int i,j,k;

		 // set dynamic constants
		 if (numPaints>12) paintWidth = 20;
		 else if (numPaints>8) paintWidth = 30;
		 else if (numPaints>4) paintWidth = 50;
		 else paintWidth = 75;
		 palHeight = (palMargin*2) + paintHeight;
		 palWidth = (palMargin*2) + (numPaints * (paintWidth + paintSpacing)) - paintSpacing;
		 pixPal = new int[(palHeight*palWidth)];
		 for(j=0;j<(palHeight*palWidth);j++) 
			 pixPal[j] = palBkgColor;
		 
		 // precompute coords of paint ovals 
		 // (can't use drawOval - colors are messed)
		 pixPaint = new int[(paintHeight*paintWidth)];
		 for(x=0;x<paintWidth;x++) {
			 for(y=0;y<paintHeight;y++) {
				 i = (int) ((y*paintWidth)+x);   

				 tx = x - (double)(paintWidth/2);
				 ty = y - (double)(paintHeight/2);
				 a = (double)(paintWidth/2);
				 b = (double)(paintHeight/2);
				 if ( (((tx*tx)/((a-paintEdge)*(a-paintEdge)))+((ty*ty)/((b-paintEdge)*(b-paintEdge)))) < 1)
					 pixPaint[i] = 1;  // outer oval
				 else if ( (((tx*tx)/(a*a))+((ty*ty)/(b*b))) < 1) 
					 pixPaint[i] = 2;  // inner oval
				 else 
					 pixPaint[i] = 0;  // outside both
			 }
		 }
		 // put frame around palette
		 for(i=0;i<palWidth;i++) {           
			 for(j=0;j<frameThickness;j++) {   
				 if (palPos=="EAST") {
					 pixPal[(i*palHeight)+j] = frameColor;               // east 
					 pixPal[(i*palHeight)+(palHeight-j-1)] = frameColor; // west
				 }
				 else {
					 pixPal[(j*palWidth)+i] = frameColor;                // north
					 pixPal[((palHeight-j-1)*palWidth)+i] = frameColor;  // south
				 }
			 }
		 }
		 for(j=0;j<palHeight;j++) {          
			 for(i=0;i<frameThickness;i++) {    
				 if (palPos=="EAST") {
					 pixPal[(i*palHeight)+j] = frameColor;               // north               
					 pixPal[((palWidth-i-1)*palHeight)+j] = frameColor;               // south
				 }
				 else {
					 pixPal[(j*palWidth)+i] = frameColor;                // east
					 pixPal[(j*palWidth)+(palWidth-i-1)] = frameColor;   // west
				 }
			 }
		 }

		 // create palette
		 palette = new int[numPaints];
		 setDynamicPalette();
		 createPaletteImage();
	 }
	 
	 
	 
	 // clearScreen()
	 // Resets the picture bit-map to original.
	 
	 private void clearScreen() {
		 int i,j,x,y;
		 
		 // set all alpha values to 255
		 for(i=0;i<imageSize;i++) 
			 pixFile[i] |= 0xFF000000;
		 
		 // create screen bit map (extract edges)
		 for(i=0;i<imageSize;i++) {
			 if (pixFile[i] == 0xFF000000) 
				 pixScreen[i] = 0xFF000000;
			 else
				 pixScreen[i] = 0xFFFFFFFF;
		 }

		 // color the frame
		 for(x=0;x<imgWidth;x++) {           
			 for(y=0;y<frameThickness;y++) {   
				 pixScreen[(y*imgWidth)+x] = frameColor2;               // north
				 pixScreen[((imgHeight-y-1)*imgWidth)+x] = frameColor2; // south
			 }
		 }
		 for(y=0;y<imgHeight;y++) {          
			 for(x=0;x<frameThickness;x++) {    
				 pixScreen[(y*imgWidth)+x] = frameColor2;               // east
				 pixScreen[(y*imgWidth)+(imgWidth-x-1)] = frameColor2;  // west
			 }
		 }
		 
		 // regenerate screen image
		 imgScreen = createImage(new MemoryImageSource(imgWidth,imgHeight,pixScreen,0,imgWidth));
	 }
	 
	private void printScreen(Graphics g) {
		 int i,j,x,y;
		 Image printPix;
		 
		 // set all alpha values to 255
		 for(i=0;i<imageSize;i++) 
			 pixFile[i] |= 0xFF000000;
		 
		 // create screen bit map (extract edges)
		 for(i=0;i<imageSize;i++) {
			 if (pixFile[i] == 0xFF000000) 
				 pixScreen[i] = 0xFF000000;
			 else
				 pixScreen[i] = 0xFFFFFFFF;
		 }

		 // color the frame
		 for(x=0;x<imgWidth;x++) {           
			 for(y=0;y<frameThickness;y++) {   
				 pixScreen[(y*imgWidth)+x] = frameColor2;               // north
				 pixScreen[((imgHeight-y-1)*imgWidth)+x] = frameColor2; // south
			 }
		 }
		 for(y=0;y<imgHeight;y++) {          
			 for(x=0;x<frameThickness;x++) {    
				 pixScreen[(y*imgWidth)+x] = frameColor2;               // east
				 pixScreen[(y*imgWidth)+(imgWidth-x-1)] = frameColor2;  // west
			 }
		 }
		 
		 // regenerate screen image
		 printPix = createImage(new MemoryImageSource(imgWidth,imgHeight,pixScreen,0,imgWidth));
		 g.drawImage(printPix,0,0,null); 
	}

	 // parseNumericalParameters
	 //
	 public void parseNumericalParameters() {
		 String value;
		 int n=0;

		 // number of paints 
		 value = this.getParameter("paints");
		 try { n = Integer.parseInt(value); }
		 catch (NumberFormatException e) { 
			 n=0;
		 }
		 if (n>3 && n<17) numPaints = n;

		 // palette mask
		 value = this.getParameter("mask");
		 try { n = Integer.parseInt(value); }
		 catch (NumberFormatException e) { 
			 n=0;
		 }
		 if (n>0) paletteMask = n;

		 // imgWidth
		 value = this.getParameter("imgwidth");
		 try { n = Integer.parseInt(value); }
		 catch (NumberFormatException e) { 
		 }
		 if (n>0) imgWidth = n;
		 
		 // imgHeight
		 value = this.getParameter("imgheight");
		 try { n = Integer.parseInt(value); }
		 catch (NumberFormatException e) { 
		 }
		 if (n>0) imgHeight = n;
		 
		 
		 
		 // background
		 value = this.getParameter("background");
		 if (value==null) value="FFFFFF";
		 try { n = Integer.parseInt(value,16); }
		 catch (NumberFormatException e) { 
			 n = Integer.parseInt("FFFFFF",16);
		 }
		 this.setBackground(new Color(n));
		 palBkgColor = (0xFF << 24) | n;
		 
		 //paletteBackground
		 value = this.getParameter("palettebackground");
		 if (value == null) value = "ffffff";
		 try { n = Integer.parseInt(value,16); }
		 catch (NumberFormatException e) { 
			 n = Integer.parseInt("ffffff",16);
		 }
		 palBkgColor = (0xFF << 24) | n;
		 
		 // foreground
		 value = this.getParameter("foreground");
		 if (value==null) value="000000";
		 try { n = Integer.parseInt(value,16); }
		 catch (NumberFormatException e) { 
			 n = Integer.parseInt("000000", 16);
		 }
		 this.setForeground(new Color(n));

		 // vertical palette
		 value = this.getParameter("palvertical");
		 try { n = Integer.parseInt(value); }
		 catch (NumberFormatException e) { 
			 n = 0;
		 }
		 palPos = "NORTH";
		 if (n>0) palPos = "EAST";
	 }

	 // initImage()
	 // Allocates all memory for images and creates the bit map which 
	 // represents the picture.
	 //
	 private void initImage() {
		 // allocate memory
		 imageSize = imgWidth*imgHeight;
		 pixFile = new int[imageSize];
		 pixScreen = new int[imageSize];
		 
		 // create bit map from file
		 PixelGrabber pg;

		 imgFile = this.getImage(this.getDocumentBase(),filename);
		 // (catch file not found exception)
		 pg = new PixelGrabber(imgFile,0,0,imgWidth,imgHeight,pixFile,0,imgWidth);
		 try {
			 pg.grabPixels();
		 } catch(Exception e) {
			 // (handle exception)
		 }
		 
		 // add frame
		 int x,y;
		 for(x=0;x<imgWidth;x++) {           
			 for(y=0;y<frameThickness;y++) {   
				 pixFile[(y*imgWidth)+x] = 0xFF000000;               // north
				 pixFile[((imgHeight-y-1)*imgWidth)+x] = 0xFF000000; // south
			 }
		 }
		 for(y=0;y<imgHeight;y++) {          
			 for(x=0;x<frameThickness;x++) {    
				 pixFile[(y*imgWidth)+x] = 0xFF000000;               // east
				 pixFile[(y*imgWidth)+(imgWidth-x-1)] = 0xFF000000;  // west
			 }
		 }
	 }

	 // init
	 //
	 public void init() {
		MediaTracker tracker = new MediaTracker(this);
		Image upImage=null,downImage=null,overImage=null;
		Image upImage2=null,downImage2=null,overImage2=null;
		String upFileName,downFileName,overFileName;
		
		upFileName = this.getParameter("xu");
		downFileName =this.getParameter("xd");
		overFileName = this.getParameter("xo");
		
		//grab the images from disk
		if (upFileName != null) {
			upImage = this.getImage(this.getDocumentBase(),upFileName);
		}
		if (downFileName != null) {
			downImage = this.getImage(this.getDocumentBase(),downFileName);
		}
		if (overFileName != null) {
			overImage = this.getImage(this.getDocumentBase(),overFileName);
		}
		if (upImage != null) {
			tracker.addImage(upImage,0);
		}
		if (downImage != null) {
			tracker.addImage(downImage,0);
		}
		if (overImage != null) {
			tracker.addImage(overImage,0);
		}
		
		try {
			tracker.waitForAll();
		} catch (InterruptedException ex) {
			System.out.println("Can't load an image");
		}
				
		 // GUI
		 more = new ImageButton(upImage,downImage,overImage);
		 buttonWidth = more.getSize().width;
		 buttonHeight = more.getSize().height;
		 more.hide();
		 more.addMouseListener(buttonAction);
		 this.add(more);
	
		upFileName = this.getParameter("zu");
		downFileName =this.getParameter("zd");
		overFileName = this.getParameter("zo");
		
		//grab the images from disk
		if (upFileName != null) {
			upImage2 = this.getImage(this.getDocumentBase(),upFileName);
		}
		if (downFileName != null) {
			downImage2 = this.getImage(this.getDocumentBase(),downFileName);
		}
		if (overFileName != null) {
			overImage2 = this.getImage(this.getDocumentBase(),overFileName);
		}
		if (upImage2 != null) {
			tracker.addImage(upImage2,0);
		}
		if (downImage2 != null) {
			tracker.addImage(downImage2,0);
		}
		if (overImage2 != null) {
			tracker.addImage(overImage2,0);
		}
		
		try {
			tracker.waitForAll();
		} catch (InterruptedException ex) {
			System.out.println("Can't load an image");
		}
		
		 clear = new ImageButton(upImage2,downImage2,overImage2);
		 clear.hide();
		 clear.addMouseListener(buttonAction);
		 this.add(clear);
		 
		 
		 
		 // applet parameters
		 parseNumericalParameters();
		 filename = this.getParameter("file");
		 selectedPalette = 8;
		 selectedPalette = nextPalette();
		 initPalette();    

		 // button box
		 boxWidth = palHeight;

	 }

	 // paint
	 //
	 public void paint(Graphics g) {

		 if (firstPass==true) {
			 Font ft = new Font("Helvetica",Font.BOLD, 24);
			 g.setFont(ft);
			 g.setColor(Color.red);
			 g.drawString(". . . Loading Image . . . Please Wait . . . ",100,200);
			 
			 // prevent unauthorized use
			 String host = new String(this.getDocumentBase().getHost().toUpperCase());
			 String valid = validHost.toUpperCase();

			 if (host.startsWith(valid)!=true && enableSecurity==true) {
				 authorized=false;
				 Font f = new Font("Helvetica",Font.BOLD, 16);
				 g.setFont(f);
				 g.drawString("Sorry, this applet will only run from " + valid,100,300);
				 System.exit(0);
			 }
			 else {
				 authorized = true;
				 int y1pos, y2pos, spacing;
				 spacing = (boxWidth-(2*buttonHeight))/3;
				 y1pos = spacing;
				 y2pos = (2*spacing) + buttonHeight;

				 // show more colors button if more than one color (except random palette)
				 int next = nextPalette();
				 if ((selectedPalette==next) && (next!=5)) more.hide();  
				 else more.show();
				 clear.show();

				 // position the buttons
				 if (palPos=="EAST") {
					 more.reshape(buttonXMargin,palWidth+y1pos,buttonWidth,buttonHeight);
					 clear.reshape(buttonXMargin,palWidth+y2pos,buttonWidth,buttonHeight);
				 }
				 else {
					 more.reshape(palWidth,0,buttonWidth,buttonHeight);
					 clear.reshape(palWidth,buttonHeight+buttonXMargin,buttonWidth,buttonHeight);
				 }
			 }

			 initImage();      // image
			 g.clearRect(0,0,this.getSize().width,this.getSize().height);
			 clearScreen();    // reset
			 firstPass = false;
			 secondPass = true;
			 return;
		 }
		 
		 if (secondPass) {
			 secondPass = false;
			 g.clearRect(0,0,this.getSize().width,this.getSize().height);
		 }
		 
		 if (authorized==true) 
			 update2(g);   // calling update prevents flicker
	 }

	 public void update(Graphics g) {
		 paint (g);
	 }
	 
	 // nextPalette
	 // Determine the next palette using the palette bit mask.
	 // 
	 public int nextPalette() {
		 int x=selectedPalette;
		 do {
			 x++;
			 if (x>5) x=0;
		 }
			 while( ((1<<x) & paletteMask) == 0);
		 return(x);
	 }



	 // updatePalette
	 //
	 public void updatePalette(Graphics g) {
		 // display palette
		 if (palPos=="EAST") 
			 g.drawImage(imgPal,0,0,palHeight,palWidth,this);
		 else 
			 g.drawImage(imgPal,0,0,palWidth,palHeight,this);
	 }



	 // updatePicture
	 //
	 public void updatePicture(Graphics g) {
		 // dispay image
		 if (palPos=="EAST") 
			 g.drawImage(imgScreen,palHeight,0,imgWidth,imgHeight,this);
		 else 
			 g.drawImage(imgScreen,0,palHeight-frameThickness,imgWidth,imgHeight,this);
	 }



	 // update
	 //
	 public void update2(Graphics g) {
		 Color box = new Color(boxColor);
		 
		 // button box
		 g.setColor(box);   
		/* if (palPos=="EAST") 
			 g.fillRoundRect(0,palWidth,boxWidth,boxWidth,20,20);
		 else 
			 g.fillRoundRect(palWidth,0,boxWidth,boxWidth,20,20);
		 
		 g.setColor(Color.black);       
		 if (palPos=="EAST")
			 g.drawRoundRect(0,palWidth,boxWidth,boxWidth,20,20);
		 else 
			 g.drawRoundRect(palWidth,0,boxWidth,boxWidth,20,20);
			*/
		 // the rest
		 updatePalette(g);
		 updatePicture(g);
	 }

	 public class doButtons extends MouseAdapter {
		 public void mouseReleased(MouseEvent e){
			 ImageButton xbut= (ImageButton)e.getSource();
			 if (xbut.equals(more)) {
				 selectedPalette = nextPalette();
			 }
			 else if (xbut.equals(clear)) {
				 clearScreen();
			 }
			 setDynamicPalette();
			 createPaletteImage();
			 updatePalette(getGraphics());
		}
	 }

	 // handleEvent
	 //
	 public boolean handleEvent(Event e) {

		 if (e.id == Event.ACTION_EVENT) {
			 if (e.target == more) {
				 selectedPalette = nextPalette();
			 }
			 else if (e.target == clear) {
				 clearScreen();
			 }
			 setDynamicPalette();
			 createPaletteImage();
			 updatePalette(this.getGraphics());
		 }

		 else if (e.id == Event.MOUSE_DOWN) {
			 int x = e.x;
			 int y = e.y;
			 int i;

			 // clicked on image
			 
			 if (((y > (palHeight+frameThickness)) && 
				  (y < (palHeight+imgHeight-frameThickness)) &&
				  (x > frameThickness) &&
				  (x < (imgWidth-frameThickness)) &&
				  palPos=="NORTH") ||

				 ((y > frameThickness) &&
				  (y < imgHeight-frameThickness) &&
				  (x > (palHeight+frameThickness)) && 
				  (x < (palHeight+imgWidth-frameThickness)) &&
				  palPos=="EAST")) {
				 
				 if (palPos=="EAST") 
					 x -= palHeight;
				 else 
					 y -= (palHeight - frameThickness);

				 if (x<0) x=0;
				 if (y<0) y=0;

				 if ((pixFile[(y*imgWidth)+x]&0x00FFFFFF)!=0) { // not an edge
					 int fill = palette[selectedColor];
					 int match = pixFile[(y*imgWidth)+x];
					 
					 // fill in screen image
					 for(i=0;i<imageSize;i++) {
						 if (pixFile[i]==match) {
							 pixScreen[i] = fill;
						 }
					 }
					 imgScreen = createImage(new MemoryImageSource(imgWidth,imgHeight,pixScreen,0,imgWidth));
				 }	 
				 updatePicture(this.getGraphics());
			 }
			 
			 // clicked on palette
			 
			 else if (((x>palMargin) && (x<(palWidth-palMargin)) &&
					   (y>palMargin) && (y<(palHeight-palMargin)) &&
					   palPos=="NORTH") ||
					  
					  ((x>palMargin) && (x<(palHeight-palMargin)) &&
					   (y>palMargin) && (y<(palWidth-palMargin))) &&
					  palPos=="EAST") {
				 
				 // take the closest color as selected
				 if (palPos=="EAST") 
					 selectedColor = (int)((y-palMargin)/(paintWidth+paintSpacing));
				 else
					 selectedColor = (int)((x-palMargin)/(paintWidth+paintSpacing));
				 createPaletteImage();
				 updatePalette(this.getGraphics());
			 }
		 }
		 return(true);
	 }
	 
	 public void print(Graphics g) {
		 g.setColor(new Color(255,255,255));
		 g.fillRect(0,0,this.getSize().width,this.getSize().height);
		 g.setColor(new Color(0,0,0));
		 g.drawString("This is a test print",100,100);
		 //g.dispose();
	 }
	 
	 public void printAll(Graphics g) {
		 g.setColor(new Color(255,255,255));
		 g.fillRect(0,0,this.getSize().width,this.getSize().height);
		 g.setColor(new Color(0,0,0));
		 printScreen(g);
		 //g.dispose();
	 }
	 
 }

class ImageButton extends Canvas {
	private Image	upImg=null,
					downImg=null,
					overImg=null,
					currentImg=null;
	private String	upFileName=null,
					downFileName=null,
					overFileName=null;
	
	public ImageButton (Image up,Image down,Image over) {
		upImg = up;
		downImg = down;
		overImg = over;
		currentImg = upImg;
		setSize(upImg.getWidth(this),upImg.getHeight(this));
		addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				if ((overImg != null) && (currentImg != downImg)) {
					currentImg = overImg;
					repaint();
				} 
			}
			public void mouseExited(MouseEvent e) {
				if (currentImg == overImg) {
					currentImg = upImg;
					repaint();
				}
			}
			public void mousePressed(MouseEvent e) {
				if (downImg != null) {
					currentImg = downImg;
					repaint();
				}
			}
			public void mouseReleased(MouseEvent e) {
				currentImg = upImg;
				if(overImg != null){
				currentImg = overImg;
				}
				repaint();
			}
		});
	}
	
	public Dimension getMinimumSize() {
		return getSize();
	}
	
	public Dimension getPreferredSize() {
		return getSize();
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint(Graphics g) {
		g.drawImage(currentImg,0,0,this);
	}
	
	
}
	
	
	